package de.fhdw.gaming.memory.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.memory.ShiftList;

/**
 * Organises a list of Optional doubles, allowing to set a maximum size for is.
 */
public class MemoryShiftListImpl implements ShiftList {

    /**
     * .
     */
    private Move<?, ?> lastMyMoveUsed;

    /**
     * .
     */
    private Move<?, ?> lastOpponentMoveUsed;

    /**
     * The number of outcomes this list can save.
     */
    private int length = 0;

    /**
     * Saves the outcomes of a player given as Optional<Double> as List.
     */
    private List<MemoryPairImpl> memoryPair = new ArrayList<MemoryPairImpl>();

    /**
     * Creates a MemoryShiftListImpl object with the default length of 0.
     */
    public MemoryShiftListImpl() {
    }

    /**
     * Creates a MemoryShiftListImpl object able to save up to length outcomes.
     * 
     * @param length
     */
    public MemoryShiftListImpl(int length) {
        this.length = length;
    }

    /**
     * Adds an outcome as Optional<Double> to the list. If the list is already full,
     * the oldest outcome will be deleted before adding the new one.
     * 
     * @param input
     */
    @Override
    public void push(Optional<Double> input) {

        final MemoryPairImpl pairToAdd = new MemoryPairImpl(input, lastMyMoveUsed, lastOpponentMoveUsed);

        if (length == -1) {
            memoryPair.add(pairToAdd);
        } else if (length < -1) {
            // throw error
        } else if (length == 0) {

        } else if (length == memoryPair.size()) {
            memoryPair.remove(0);
            memoryPair.add(pairToAdd);
        } else if (length > memoryPair.size()) {
            memoryPair.add(pairToAdd);
        }
    }

    /**
     * Adds an outcome as Optional<Double> to the list. If the list is already full,
     * the oldest outcome will be deleted before adding the new one.
     * 
     * @param moveUsed
     */
    public void pushMyMove(Move<?, ?> moveUsed) {
        lastMyMoveUsed = moveUsed;
    }

    /**
     * Adds an outcome as Optional<Double> to the list. If the list is already full,
     * the oldest outcome will be deleted before adding the new one.
     * 
     * @param moveUsed
     */
    public void pushOpponentMove(Move<?, ?> moveUsed) {
        lastOpponentMoveUsed = moveUsed;
    }

    /**
     * Returns the list of all recorded outcomes as Optional<Double>.
     */
    @Override
    public List<MemoryPairImpl> show() {
        if (!this.memoryPair.isEmpty()) {
            return this.memoryPair;
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * Determines how many outcomes can be saved in the list displyyed by the show()
     * method. If there are too many outcomes saved this will delete the oldest
     * outcome until the number of outcomes is equal to the length.
     * 
     * @param length
     */
    @Override
    public void setLength(int length) {
        this.length = length;
        if (length < -1) {
            throw new IllegalArgumentException(
                    "The length value entered cannot be used. The value must be >= -1.");
        }
        while (length < memoryPair.size() && length != -1) {
            memoryPair.remove(0);
        }
    }

    /**
     * Returns the number of outcomes that can be saves.
     */
    @Override
    public int getLength() {
        return length;
    }

}
